@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2004 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2010 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -12,6 +12,8 @@
 #include	<stdio.h>
 #include	<errno.h>
 #include	<string.h>
+#include	<langinfo.h>
+
 #if	HAVE_STRINGS_H
 #include	<strings.h>
 #endif
@@ -29,6 +31,8 @@
 #include	"rfc822/rfc822.h"
 #include	"rfc822/rfc2047.h"
 #include	"rfc2045charset.h"
+#include	"unicode/unicode.h"
+
 #if HAVE_UNISTD_H
 #include	<unistd.h>
 #endif
@@ -44,7 +48,9 @@ int gethostname(const char *, size_t);
 
 extern int rfc2045_in_reformime;
 
-static const char rcsid[]="$Id: reformime.c,v 1.47 2004/04/11 17:35:35 mrsam Exp $";
+static const struct unicode_info *uniinfo;
+
+static const char rcsid[]="$Id: reformime.c,v 1.52 2010/02/15 14:27:34 mrsam Exp $";
 
 void rfc2045_error(const char *errmsg)
 {
@@ -149,8 +155,7 @@ int	l;
 
 	if (tempfp)	/* Replace stdin */
 	{
-		close(0);
-		dup(fileno(tempfp));
+		dup2(fileno(tempfp), 0);
 		fclose(tempfp);
 	}
 	return (p);
@@ -223,11 +228,15 @@ char *disposition_name, *disposition_filename;
 		printf("content-id: <%s>\n", p);
 	if (*(p=rfc2045_content_description(s)))
 	{
-	char *s=rfc2047_decode_simple(p);
+		char *s=rfc822_display_hdrvalue_tobuf("content-description",
+						      p,
+						      uniinfo->chset,
+						      NULL,
+						      NULL);
 
 		if (!s)
 		{
-			perror("rfc2047_decode_simple");
+			perror("rfc2047_decode_unicode");
 			exit(1);
 		}
 		printf("content-description: %s\n", s);
@@ -363,7 +372,7 @@ const char *disposition_filename_s;
 
 	if (!filename_buf)
 	{
-		perror("rfc2047_decode_simple");
+		perror("strdup");
 		exit(1);
 	}
 
@@ -474,7 +483,9 @@ const char *disposition_filename_s;
 			return (0);
 		}
 		fprintf (tty, "Filename [%s]: ", p);
-		fgets(filenamebuf, sizeof(filenamebuf)-1, tty);
+		if (fgets(filenamebuf, sizeof(filenamebuf)-1, tty) == NULL)
+			filenamebuf[0]=0;
+
 		fclose(tty);
 		t=strchr(filenamebuf, '\n');
 		if (t)	*t=0;
@@ -597,8 +608,7 @@ int	waitstat;
 		}
 		strcat(strcpy(f, "CONTENT_TYPE="), content_type_s);
 		putenv(f);
-		close(0);
-		dup(pipefd[0]);
+		dup2(pipefd[0], 0);
 		close(pipefd[0]);
 		close(pipefd[1]);
 		execv(argv[0], argv);
@@ -911,17 +921,24 @@ FILE	*fp;
 	printf("\n--%s--\n", boundarybuf);
 }
 
-int main(int argc, char **argv)
+static void display_decoded_header(const char *ptr, size_t cnt, void *dummy)
+{
+	if (cnt == 0)
+		putchar('\n');
+	else
+		fwrite(ptr, cnt, 1, stdout);
+}
+
+static int main2(const char *mimecharset, int argc, char **argv)
 {
 int	argn;
 char	optc;
 char	*optarg;
 char	*mimesection=0;
 int	doinfo=0, dodecode=0, dorewrite=0, dodsn=0, domimedigest=0;
-int	dodecodehdr=0, doencodemime=0, doencodemimehdr=0;
+int	dodecodehdr=0, dodecodeaddrhdr=0, doencodemime=0, doencodemimehdr=0;
 
 char	*decode_header="";
- char	*mimecharset="iso-8859-1";
 struct	rfc2045 *p;
 int	rwmode=0;
 int	dovalidate=0;
@@ -929,9 +946,6 @@ void	(*do_extract)(struct rfc2045 *, const char *, int, char **)=0;
 const char *extract_filename=0;
 int rc=0;
 
-#if HAVE_SETLOCALE
-	setlocale(LC_ALL, "C");
-#endif
 
 	rfc2045_in_reformime=1;
 
@@ -949,7 +963,12 @@ int rc=0;
 			if (optarg && *optarg)
 			{
 				mimecharset=optarg;
-				rfc2045_setdefaultcharset(optarg);
+				if (unicode_find(mimecharset) == NULL)
+				{
+					fprintf(stderr, "Unknown charset: %s\n",
+						mimecharset);
+					exit(1);
+				}
 			}
 			break;
 
@@ -1003,6 +1022,15 @@ int rc=0;
 			}
 			dodecodehdr=1;
 			break;
+		case 'H':
+			if (!optarg && argn < argc)
+				optarg=argv[argn++];
+			if (optarg)
+			{
+				decode_header=optarg;
+			}
+			dodecodeaddrhdr=1;
+			break;
 		case 'o':
 			if (!optarg && argn < argc)
 				optarg=argv[argn++];
@@ -1027,6 +1055,13 @@ int rc=0;
 		}
 	}
 
+	uniinfo=unicode_find(mimecharset);
+
+	if (!uniinfo)
+		uniinfo=&unicode_ISO8859_1;
+
+	rfc2045_setdefaultcharset(uniinfo->chset);
+
 	if (domimedigest)
 	{
 		mimedigest(argc-argn, argv+argn);
@@ -1034,15 +1069,37 @@ int rc=0;
 	}
 	else if (dodecodehdr)
 	{
-		char *s=rfc2047_decode_simple(decode_header);
-
-		if (s)
+		if (rfc822_display_hdrvalue("Subject",
+					    decode_header,
+					    mimecharset,
+					    display_decoded_header,
+					    NULL,
+					    NULL) < 0)
 		{
-			printf("%s\n", s);
-			free(s);
+			perror("rfc822_display_hdrvalue");
+			return (1);
 		}
+
+		printf("\n");
 		return (0);
 	}
+	else if (dodecodeaddrhdr)
+	{
+		if (rfc822_display_hdrvalue("To",
+					    decode_header,
+					    mimecharset,
+					    display_decoded_header,
+					    NULL,
+					    NULL) < 0)
+		{
+			perror("rfc822_display_hdrvalue");
+			return (1);
+		}
+
+		printf("\n");
+		return (0);
+	}
+
 	if (doencodemime)
 	{
 		char *s=rfc2047_encode_str(decode_header, mimecharset,
@@ -1061,7 +1118,7 @@ int rc=0;
 		struct rfc822a *a=t ? rfc822a_alloc(t):NULL;
 		char *s;
 
-		if (a && (s=rfc2047_encode_header(a, mimecharset)) != NULL)
+		if (a && (s=rfc2047_encode_header_addr(a, mimecharset)) != NULL)
 		{
 			printf("%s\n", s);
 			free(s);
@@ -1111,3 +1168,23 @@ int rc=0;
 	exit(rc);
 	return (rc);
 }
+
+int main(int argc, char **argv)
+{
+	char	*mimecharset;
+	int	rc;
+
+	setlocale(LC_ALL, "");
+	mimecharset=strdup(nl_langinfo(CODESET));
+	setlocale(LC_ALL, "C");
+
+	if (!mimecharset)
+	{
+		perror("malloc");
+		exit(1);
+	}
+
+	rc=main2(mimecharset, argc, argv);
+	free(mimecharset);
+	return rc;
+}
